from ..vf3py_base import calc_noattrs, calc_nodeattr, calc_edgeattr, calc_bothattrs
from ..vf3py_vf3l import calc_l_noattrs, calc_l_nodeattr, calc_l_edgeattr, calc_l_bothattrs
from ..vf3py_vf3p import calc_p_noattrs, calc_p_nodeattr, calc_p_edgeattr, calc_p_bothattrs
